@@ -13,6 +13,7 @@ import (
 	"time"
 
 	"github.com/prometheus/client_golang/prometheus"
+
 	"go.uber.org/zap"
 
 	"github.com/go-graphite/go-carbon/api"
@@ -177,6 +178,7 @@ func (app *App) ReloadConfig() error {
 	app.Cache.SetMaxSize(app.Config.Cache.MaxSize)
 	app.Cache.SetWriteStrategy(app.Config.Cache.WriteStrategy)
 	app.Cache.SetTagsEnabled(app.Config.Tags.Enabled)
+	app.Cache.SetBloomSize(app.Config.Cache.BloomSize)
 
 	if app.Persister != nil {
 		app.Persister.Stop()
@@ -330,7 +332,7 @@ func (app *App) startPersister() {
 }
 
 // Start starts
-func (app *App) Start(version string) (err error) {
+func (app *App) Start() (err error) {
 	app.Lock()
 	defer app.Unlock()
 
@@ -348,6 +350,7 @@ func (app *App) Start(version string) (err error) {
 	core.SetMaxSize(conf.Cache.MaxSize)
 	core.SetWriteStrategy(conf.Cache.WriteStrategy)
 	core.SetTagsEnabled(conf.Tags.Enabled)
+	core.SetBloomSize(conf.Cache.BloomSize)
 
 	app.Cache = core
 
@@ -488,6 +491,7 @@ func (app *App) Start(version string) (err error) {
 		carbonserver.SetFailOnMaxGlobs(conf.Carbonserver.FailOnMaxGlobs)
 		carbonserver.SetMaxMetricsGlobbed(conf.Carbonserver.MaxMetricsGlobbed)
 		carbonserver.SetMaxMetricsRendered(conf.Carbonserver.MaxMetricsRendered)
+		carbonserver.SetMaxFetchDataGoroutines(conf.Carbonserver.MaxFetchDataGoroutines)
 		carbonserver.SetBuckets(conf.Carbonserver.Buckets)
 		carbonserver.SetMetricsAsCounters(conf.Carbonserver.MetricsAsCounters)
 		carbonserver.SetScanFrequency(conf.Carbonserver.ScanFrequency.Value())
@@ -497,9 +501,12 @@ func (app *App) Start(version string) (err error) {
 		carbonserver.SetIdleTimeout(conf.Carbonserver.IdleTimeout.Value())
 		carbonserver.SetWriteTimeout(conf.Carbonserver.WriteTimeout.Value())
 		carbonserver.SetQueryCacheEnabled(conf.Carbonserver.QueryCacheEnabled)
+		carbonserver.SetQueryCacheSizeMB(conf.Carbonserver.QueryCacheSizeMB)
 		carbonserver.SetStreamingQueryCacheEnabled(conf.Carbonserver.StreamingQueryCacheEnabled)
 		carbonserver.SetFindCacheEnabled(conf.Carbonserver.FindCacheEnabled)
-		carbonserver.SetQueryCacheSizeMB(conf.Carbonserver.QueryCacheSizeMB)
+		carbonserver.SetFindCacheSizeMB(conf.Carbonserver.FindCacheSizeMB)
+		carbonserver.SetGlobCacheEnabled(conf.Carbonserver.GlobCacheEnabled)
+		carbonserver.SetGlobCacheSizeMB(conf.Carbonserver.GlobCacheSizeMB)
 		carbonserver.SetTrigramIndex(conf.Carbonserver.TrigramIndex)
 		carbonserver.SetTrieIndex(conf.Carbonserver.TrieIndex)
 		carbonserver.SetConcurrentIndex(conf.Carbonserver.ConcurrentIndex)
@@ -550,7 +557,7 @@ func (app *App) Start(version string) (err error) {
 				return logicalSize, physicalSize, dataPoints
 			})
 
-			carbonserver.SetQuotas(app.Config.getCarbonserverQuotas())
+			carbonserver.SetQuotas(app.Config.getCarbonserverQuotas(conf.Carbonserver.QuotaUsageReportFrequency.Value()))
 			core.SetThrottle(carbonserver.ShouldThrottleMetric)
 		}
 
@@ -580,7 +587,7 @@ func (app *App) Start(version string) (err error) {
 			carbonserver.InitPrometheus(app.PromRegisterer)
 		}
 		if conf.Tracing.Enabled {
-			app.FlushTraces = carbonserver.InitTracing(conf.Tracing.JaegerEndpoint, conf.Tracing.Stdout, version, conf.Tracing.SendTimeout.Value())
+			log.Printf("Otel tracing is removed in current verion, ignoring tracing.enabled=true")
 		}
 
 		carbonserver.RegisterInternalInfoHandler("cache", core.GetInfo)
