@@ -41,12 +41,13 @@ import (
 	"time"
 
 	prom "github.com/prometheus/client_golang/prometheus"
-	"go.uber.org/zap"
 	"google.golang.org/grpc/codes"
 	_ "google.golang.org/grpc/encoding/gzip"
 	"google.golang.org/grpc/peer"
 	"google.golang.org/grpc/status"
 
+	"go.uber.org/zap"
+
 	"github.com/NYTimes/gziphandler"
 	"github.com/dgryski/go-expirecache"
 	"github.com/dgryski/go-trigram"
@@ -208,11 +209,11 @@ func (q *QueryItem) StoreAndUnlock(data interface{}) {
 	close(q.QueryFinished)
 }
 
-type queryCache struct {
+type expireCache struct {
 	ec *expirecache.Cache
 }
 
-func (q *queryCache) getQueryItem(k string, size uint64, expire int32) *QueryItem {
+func (q *expireCache) getQueryItem(k string, size uint64, expire int32) *QueryItem {
 	emptyQueryItem := &QueryItem{QueryFinished: make(chan struct{})}
 	return q.ec.GetOrSet(k, emptyQueryItem, size, expire).(*QueryItem)
 }
@@ -245,16 +246,20 @@ type CarbonserverListener struct {
 	compressed        bool
 	removeEmptyFile   bool
 
-	maxMetricsGlobbed  int
-	maxMetricsRendered int
+	maxMetricsGlobbed      int
+	maxMetricsRendered     int
+	maxFetchDataGoroutines int
 
 	queryCacheEnabled          bool
 	streamingQueryCacheEnabled bool
 	queryCacheSizeMB           int
-	queryCache                 queryCache
+	queryCache                 expireCache
 	findCacheEnabled           bool
-	findCache                  queryCache
-	expandedGlobsCache         queryCache // TODO: rename queryCache type to be more generic
+	findCacheSizeMB            int
+	findCache                  expireCache
+	globCacheEnabled           bool
+	globCacheSizeMB            int
+	globCache                  expireCache
 	trigramIndex               bool
 	trieIndex                  bool
 	concurrentIndex            bool
@@ -473,15 +478,15 @@ type fileIndex struct {
 func NewCarbonserverListener(cacheGetFunc func(key string) []points.Point) *CarbonserverListener {
 	return &CarbonserverListener{
 		// Config variables
-		metrics:            &metricStruct{},
-		metricsAsCounters:  false,
-		cacheGet:           cacheGetFunc,
-		logger:             zapwriter.Logger("carbonserver"),
-		accessLogger:       zapwriter.Logger("access"),
-		findCache:          queryCache{ec: expirecache.New(0)},
-		expandedGlobsCache: queryCache{ec: expirecache.New(0)},
-		trigramIndex:       true,
-		percentiles:        []int{100, 99, 98, 95, 75, 50},
+		metrics:           &metricStruct{},
+		metricsAsCounters: false,
+		cacheGet:          cacheGetFunc,
+		logger:            zapwriter.Logger("carbonserver"),
+		accessLogger:      zapwriter.Logger("access"),
+		findCache:         expireCache{ec: expirecache.New(0)},
+		globCache:         expireCache{ec: expirecache.New(0)},
+		trigramIndex:      true,
+		percentiles:       []int{100, 99, 98, 95, 75, 50},
 		prometheus: prometheus{
 			request:          func(string, int) {},
 			duration:         func(time.Duration) {},
@@ -521,6 +526,9 @@ func (listener *CarbonserverListener) SetMaxMetricsGlobbed(max int) {
 func (listener *CarbonserverListener) SetMaxMetricsRendered(max int) {
 	listener.maxMetricsRendered = max
 }
+func (listener *CarbonserverListener) SetMaxFetchDataGoroutines(max int) {
+	listener.maxFetchDataGoroutines = max
+}
 func (listener *CarbonserverListener) SetFLock(flock bool) {
 	listener.flock = flock
 }
@@ -560,15 +568,24 @@ func (listener *CarbonserverListener) SetMetricsAsCounters(metricsAsCounters boo
 func (listener *CarbonserverListener) SetQueryCacheEnabled(enabled bool) {
 	listener.queryCacheEnabled = enabled
 }
-func (listener *CarbonserverListener) SetStreamingQueryCacheEnabled(enabled bool) {
-	listener.streamingQueryCacheEnabled = enabled
-}
 func (listener *CarbonserverListener) SetQueryCacheSizeMB(size int) {
 	listener.queryCacheSizeMB = size
 }
+func (listener *CarbonserverListener) SetStreamingQueryCacheEnabled(enabled bool) {
+	listener.streamingQueryCacheEnabled = enabled
+}
 func (listener *CarbonserverListener) SetFindCacheEnabled(enabled bool) {
 	listener.findCacheEnabled = enabled
 }
+func (listener *CarbonserverListener) SetFindCacheSizeMB(size int) {
+	listener.findCacheSizeMB = size
+}
+func (listener *CarbonserverListener) SetGlobCacheEnabled(enabled bool) {
+	listener.globCacheEnabled = enabled
+}
+func (listener *CarbonserverListener) SetGlobCacheSizeMB(size int) {
+	listener.globCacheSizeMB = size
+}
 func (listener *CarbonserverListener) SetTrigramIndex(enabled bool) {
 	listener.trigramIndex = enabled
 }
@@ -1785,7 +1802,13 @@ func (listener *CarbonserverListener) Listen(listen string) error {
 		listener.forceScanChan <- struct{}{}
 	}
 
-	listener.queryCache = queryCache{ec: expirecache.New(uint64(listener.queryCacheSizeMB))}
+	listener.queryCache = expireCache{ec: expirecache.New(uint64(listener.queryCacheSizeMB))}
+	if listener.findCacheEnabled {
+		listener.findCache = expireCache{ec: expirecache.New(uint64(listener.findCacheSizeMB))}
+	}
+	if listener.globCacheEnabled {
+		listener.globCache = expireCache{ec: expirecache.New(uint64(listener.globCacheSizeMB))}
+	}
 
 	// +1 to track every over the number of buckets we track
 	listener.timeBuckets = make([]uint64, listener.buckets+1)
@@ -1919,7 +1942,14 @@ func (listener *CarbonserverListener) Listen(listen string) error {
 		}
 	}
 
+	// cache cleaners
 	go listener.queryCache.ec.StoppableApproximateCleaner(10*time.Second, listener.exitChan)
+	if listener.findCacheEnabled {
+		go listener.findCache.ec.StoppableApproximateCleaner(60*time.Second, listener.exitChan)
+	}
+	if listener.globCacheEnabled {
+		go listener.globCache.ec.StoppableApproximateCleaner(60*time.Second, listener.exitChan)
+	}
 
 	srv := &http.Server{
 		Handler:      gziphandler.GzipHandler(carbonserverMux),
@@ -2223,7 +2253,7 @@ func (listener *CarbonserverListener) grpcServerRatelimitHandler(ctx context.Con
 	return nil
 }
 
-func getWithCache(logger *zap.Logger, cache queryCache, key string, size uint64, expire int32, f func() (interface{}, error)) (result interface{}, fromCache bool, err error) {
+func getWithCache(logger *zap.Logger, cache expireCache, key string, size uint64, expire int32, f func() (interface{}, error)) (result interface{}, fromCache bool, err error) {
 	item := cache.getQueryItem(key, size, expire)
 	res, ok := item.FetchOrLock()
 	switch {
