@@ -10,25 +10,26 @@ import (
 	"math"
 	"net/http"
 	_ "net/http/pprof" // skipcq: GO-S2108
+	"runtime"
 	"strconv"
 	"strings"
+	"sync"
 	"sync/atomic"
 	"time"
 
-	"go.uber.org/zap"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/peer"
 	"google.golang.org/grpc/status"
 	"google.golang.org/protobuf/encoding/protojson"
 
+	"go.uber.org/zap"
+
 	"github.com/go-graphite/carbonzipper/zipper/httpHeaders"
 	"github.com/go-graphite/go-whisper"
 	grpcv2 "github.com/go-graphite/protocol/carbonapi_v2_grpc"
 	protov2 "github.com/go-graphite/protocol/carbonapi_v2_pb"
 	protov3 "github.com/go-graphite/protocol/carbonapi_v3_pb"
 	pickle "github.com/lomik/og-rek"
-	"go.opentelemetry.io/otel/api/kv"
-	"go.opentelemetry.io/otel/api/trace"
 )
 
 type fetchResponse struct {
@@ -287,15 +288,6 @@ func (listener *CarbonserverListener) renderHandler(wr http.ResponseWriter, req
 		zap.Int("memory_used_bytes", response.memoryUsed),
 		zap.Int("http_code", http.StatusOK),
 	)
-
-	span := trace.SpanFromContext(ctx)
-	span.SetAttributes(
-		kv.Bool("graphite.from_cache", fromCache),
-		kv.Int("graphite.metrics", response.metricsFetched),
-		kv.Int("graphite.values", response.valuesFetched),
-		kv.Int("graphite.memory_used", response.memoryUsed),
-	)
-
 }
 
 func (listener *CarbonserverListener) getRenderCacheKeyAndSize(targets map[timeRange][]target, format string) (string, uint64) {
@@ -417,12 +409,19 @@ func (listener *CarbonserverListener) prepareDataStream(ctx context.Context, for
 					listener.accessLogger.Error(
 						"rendering too many metrics",
 						zap.Int("limit", listener.maxMetricsRendered),
-						zap.Int("target", len(files)),
+						zap.Int("files", len(files)),
+						zap.Int("leafs", len(leafs)),
+						zap.Int("trieNodes", len(trieNodes)),
 					)
-
-					files = files[:listener.maxMetricsRendered]
-					leafs = leafs[:listener.maxMetricsRendered]
-					trieNodes = trieNodes[:listener.maxMetricsRendered]
+					if len(files) > listener.maxMetricsRendered {
+						files = files[:listener.maxMetricsRendered]
+					}
+					if len(leafs) > listener.maxMetricsRendered {
+						leafs = leafs[:listener.maxMetricsRendered]
+					}
+					if len(trieNodes) > listener.maxMetricsRendered {
+						trieNodes = trieNodes[:listener.maxMetricsRendered]
+					}
 				}
 
 				metricsCount := 0
@@ -597,26 +596,46 @@ func (listener *CarbonserverListener) prepareDataProto(ctx context.Context, logg
 func (listener *CarbonserverListener) fetchData(metric, pathExpression string, files []string, leafs []bool, trieNodes []*trieNode, fromTime, untilTime int32) ([]response, error) {
 	var multi []response
 	var errs []error
-	for i, fileName := range files {
-		if !leafs[i] {
-			listener.logger.Debug("skipping directory", zap.String("PathExpression", pathExpression),
-				zap.String("metricName", metric), zap.String("fileName", fileName))
-			// can't fetch a directory
-			continue
-		}
-		response, err := listener.fetchSingleMetric(fileName, pathExpression, fromTime, untilTime)
-		if err == nil {
-			multi = append(multi, response)
-			if listener.trieIndex {
-				readBytesNumber := int64(len(response.Values) * whisper.PointSize) // bytes read from the disc, 12 bytes for each point
-				if len(trieNodes) > i {
-					trieNodes[i].incrementReadBytesMetric(readBytesNumber)
-				}
-			}
-		} else {
-			errs = append(errs, err)
-		}
+	var wg sync.WaitGroup
+	var mu sync.Mutex
+	var maxFetchDataGoroutines int
+	// listener.maxFetchDataGoroutines is the maximum number of goroutines you want to allow
+	// but < 0 is always wrong
+	if listener.maxFetchDataGoroutines > 0 {
+		maxFetchDataGoroutines = listener.maxFetchDataGoroutines
+	} else {
+		maxFetchDataGoroutines = 2 * runtime.GOMAXPROCS(0)
 	}
+	sem := make(chan struct{}, maxFetchDataGoroutines)
+	for i, fileName := range files {
+		wg.Add(1)
+		sem <- struct{}{} // acquire a slot
+		go func(i int, fileName string) {
+			defer wg.Done()
+			defer func() { <-sem }() // release the slot
+			if !leafs[i] {
+				listener.logger.Debug("skipping directory", zap.String("PathExpression", pathExpression),
+					zap.String("metricName", metric), zap.String("fileName", fileName))
+				// can't fetch a directory
+				return
+			}
+			response, err := listener.fetchSingleMetric(fileName, pathExpression, fromTime, untilTime)
+			mu.Lock()
+			defer mu.Unlock()
+			if err == nil {
+				multi = append(multi, response)
+				if listener.trieIndex {
+					readBytesNumber := int64(len(response.Values) * whisper.PointSize) // bytes read from the disc, 12 bytes for each point
+					if len(trieNodes) > i {
+						trieNodes[i].incrementReadBytesMetric(readBytesNumber)
+					}
+				}
+			} else {
+				errs = append(errs, err)
+			}
+		}(i, fileName)
+	}
+	wg.Wait()
 	if len(errs) != 0 {
 		listener.logger.Warn("errors occur while fetching data",
 			zap.Any("errors", errs),
@@ -830,13 +849,6 @@ func (listener *CarbonserverListener) Render(req *protov2.MultiFetchRequest, str
 		zap.Int("http_code", http.StatusOK),
 	)
 
-	span := trace.SpanFromContext(ctx)
-	span.SetAttributes(
-		kv.Bool("graphite.from_cache", fromCache),
-		kv.Int("graphite.metrics", metricsFetched),
-		kv.Int("graphite.values", valuesFetched),
-	)
-
 	return nil
 }
 
